import re
import matplotlib.pyplot as plt
import numpy as np


plt.rcParams.update({"font.size": 18, "font.family": "serif"})


def extract_data(line):
    match = re.search(r"Iteration (\d+) Loss (\d+\.\d+) PSNR_noisy: (\d+\.\d+) psnr_gt: (\d+\.\d+) PSNR_gt_sm: (\d+\.\d+)", line)
    if match:
        return tuple(map(float, match.groups()))
    return None


# log_file = "deep_prior.txt"
# log_file = "tcnn_1.3.txt"
# log_file = "tcnn_1.4.txt"
# log_file = "tcnn_1.5.txt"
# log_file = "tcnn_freq.txt"
log_file = "tcnn_none.txt"

data_lists = []
current_list = []
i = 0

with open(log_file, "r") as file:
    for line in file:
        if line.startswith("Iteration"):
            data = extract_data(line)
            if data:
                iteration, loss, psnr_noisy, psnr_gt, psnr_gt_sm = data
                current_list.append((iteration, loss, psnr_noisy, psnr_gt, psnr_gt_sm))
                if iteration == 990:
                    # if iteration == 990:
                    data_lists.append(current_list)
                    current_list = []
                    i += 1

if current_list:
    data_lists.append(current_list)

# Group data_lists into chunks of 15
grouped_data_lists = [data_lists[i : i + 15] for i in range(0, len(data_lists), 15)]

plt.figure(figsize=(18, 6))
sigmas = [15, 25, 50]

for i, group in enumerate(grouped_data_lists, start=1):
    plt.subplot(1, 3, i)

    for data_list in group:
        iterations, losses, psnr_noisys, psnr_gts, psnr_gt_sms = zip(*data_list)
        plt.plot(iterations, psnr_gt_sms, color="#CB7E83", alpha=0.1, linewidth=2)
        plt.plot(iterations, psnr_gts, color="#95BAA6", alpha=0.1, linewidth=2)
        plt.plot(iterations, psnr_noisys, color="#76A2BB", alpha=0.1, linewidth=2)

    group_array = np.array(group)
    avg_psnr_gt_sms = group_array.mean(axis=0)[:, 4]
    print(max(avg_psnr_gt_sms))
    print(np.argmax(avg_psnr_gt_sms))
    avg_iterations = group_array[0, :, 0]
    plt.plot(avg_iterations, avg_psnr_gt_sms, color="#CB7E83", linewidth=2, label=f"Denoised Average")

    avg_psnr_gts = group_array.mean(axis=0)[:, 3]
    print(max(avg_psnr_gts))
    print(np.argmax(avg_psnr_gts))
    plt.plot(avg_iterations, avg_psnr_gts, color="#95BAA6", linewidth=2, label=f"Denoised")

    avg_psnr_noisys = group_array.mean(axis=0)[:, 2]
    plt.plot(avg_iterations, avg_psnr_noisys, color="#76A2BB", linewidth=2, label=f"Noisy")

    plt.xlabel("Iteration")
    plt.ylabel("PNSR")
    plt.title(f"Instant-NGP PSNR (sigma={sigmas[i-1]})")
    # plt.title(f"Deep Prior PSNR (sigma={sigmas[i-1]})")
    plt.legend()

plt.tight_layout()

# plt.savefig("tcnnfreq.png", bbox_inches="tight", dpi=200)
# plt.savefig("tcnn1.3.png", bbox_inches="tight", dpi=200)
# plt.savefig("tcnn1.4.png", bbox_inches="tight", dpi=200)
# plt.savefig("tcnn1.5.png", bbox_inches="tight", dpi=200)
# plt.savefig("deep_prior.png", bbox_inches="tight", dpi=200)
plt.savefig("tcnn_none.png", bbox_inches="tight", dpi=200)
